//----------------------------------------------------------------------------------------------

// Copy3DJob.cp

//----------------------------------------------------------------------------------------------

/*


*/

#include "Copy3DJob.h"
#include "CopyUtils.h"
#include "OpenClipDefines.h"

//----------------------------------------------------------------------------------------------

Copy3DJob::Copy3DJob(DWindow *TheWndw)
  : CopyJob(TheWndw)
{
};


void Copy3DJob::HandleZHeight(const PDObject &TheObj,NSMutableDictionary *ThisObj) const
{
  PDObject TempObj = TheObj;
  
  double Z;
  if (HasAZElevation(&TempObj, Z)) {
    Z /= mScale;
    SetObjectZ(Z,ThisObj);
  };
};


void Copy3DJob::CreateXYZPointObj(CDPoint ThePt,double ObjZ,NSMutableDictionary *ThisObj) const
{
  SetObjectType(OpenClip_PointObj,ThisObj);
  
  RGBColor kLightBrown = {50372,48573,32639};
  double ThisAlpha = 1.0;
  RGBColor kLightBlue = {32896,65535,65535};
  
  if (ObjZ > 0.0)
    SetPenColor(kLightBrown,ThisAlpha,ThisObj);
  else if (ObjZ == 0.0)
    SetPenColor(kLightBlue,ThisAlpha,ThisObj);
  else if (ObjZ < 0.0)
    SetPenColor(kBlueRGB,ThisAlpha,ThisObj);
  
  ObjZ /= mScale;
  SetObjectZ(ObjZ,ThisObj);
  
  CDPoint A = CorrectPoint(ThePt);
  
  NSMutableArray *Points = [NSMutableArray array];
  AddXYPoint(A.x,A.y,Points);
  AddPointsToObject(Points,ThisObj);
};


void Copy3DJob::CreatePointObj(const PDObject &TheObj,NSMutableDictionary *ThisObj) const
{
  SetObjectType(OpenClip_PointObj,ThisObj);
    
  PDObject TempObj = TheObj;
  double ObjZ;
  if (HasAZElevation(&TempObj, ObjZ)) {
    ObjZ /= mScale;
    SetObjectZ(ObjZ,ThisObj);
    
    RGBColor kLightBrown = {50372,48573,32639};
    RGBColor kLightBlue = {32896,65535,65535};
    double ThisAlpha = 1.0;
    
    if (ObjZ > 0.0)
      SetPenColor(kLightBrown,ThisAlpha,ThisObj);
    else if (ObjZ == 0.0)
      SetPenColor(kLightBlue,ThisAlpha,ThisObj);
    else if (ObjZ < 0.0)
      SetPenColor(kBlueRGB,ThisAlpha,ThisObj);
      
  }
  else {
    RGBColor ThePenColor = *TheObj.GetPenColor();
    double PenAlpha = TheObj.GetPenAlpha();
    SetPenColor(ThePenColor,PenAlpha,ThisObj);
  };
  
  CDPoint A = TheObj.DataPoint();
  A = CorrectPoint(A);
  
  NSMutableArray *Points = [NSMutableArray array];
  AddXYPoint(A.x,A.y,Points);
  AddPointsToObject(Points,ThisObj);
};



void Copy3DJob::AddObjectToList(const PDObject &TheObj,NSMutableArray *TheNSList) const
{
  CDPoint ThePt;
  double  ObjZ;
  
  if (IsATopoObject(&TheObj, ThePt, ObjZ)) {
    NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
    CreateXYZPointObj(ThePt,ObjZ,ThisObj);
    
    [TheNSList addObject:ThisObj]; 
  }
  else {
  
    switch (TheObj.WhatObject()) {
      case pointObj   : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          CreatePointObj(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj]; 
                        }
                        break;
      case lineObj    : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          CreateLineObj(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj]; 
                        }
                        break;
      case arcObj     :
      case rtArcObj   : 
      case ovalObj    : 
      case rOvalObj   : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          CreateEllipseObj(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj]; 
                        }
                        break;
      case rectObj    : 
      case rtRectObj  :
      case rrectObj   : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          CreateRectangleObj(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj]; 
                        }
                        break;
      case polyObj    : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          CreatePolygonObj(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj];
                        };
                        break;
      case smoothObj  : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          
                          PDObject TheBez;
                          BSplineToBezier(TheObj, TheBez);
                          CreateBezierObj(TheBez,ThisObj);
                          RecoverObject( TheBez);
                          
                          AddBSplineAsAdvanced(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj]; 
                        }
                        break;
      case bezObj     : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          CreateBezierObj(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj]; 
                        }
                        break;
      case grpObj     :
      case xtnlGrpObj : {
                          NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
                          CreateGroupObj(TheObj,ThisObj);
                          
                          [TheNSList addObject:ThisObj]; 
                        }
                        break;
    };
  };

};



void Copy3DJob::CheckList(ListRecord TheList,bool InAttachObj,NSMutableArray *TheNSList) const
{
  CPDListIter	Iter(TheList,false,VectorObjects);
  	
	ObjSpec			TheObjS;
	
  while (Iter.Next( TheObjS)) {
	  PDObject TheObj = TheObjS;
		
    CDPoint ThePt;
    double  ObjZ;
    
    if (IsATopoObject(&TheObj, ThePt, ObjZ)) {  // topo point objects are special types of
      AddObjectToList(TheObj,TheNSList);        // external objects, one for WildTools/TopoTools
    }                                           // and one for Paolo Rossi's Topography Tools
    else {                                      // so they need special treatment.
      
      switch (TheObj.WhatObject()) {
        case pointObj   : 
        case lineObj    : 
        case rectObj    :
        case rtRectObj  :
        case rrectObj   :
        case rtArcObj   :
        case arcObj     :
        case ovalObj    :
        case rOvalObj   :
        case polyObj    :
        case smoothObj  :
        case bezObj     : if (InAttachObj) {
                            
                            if (TheObj.GetPenPattern() != kNonePat())
                              AddObjectToList(TheObj,TheNSList);
                          
                          }
                          else
                            AddObjectToList(TheObj,TheNSList);
                          break;
        case grpObj     :
        case xtnlGrpObj : if (mHandleGroups)
                            AddObjectToList(TheObj,TheNSList);
                          else {                       // do something about attach objects
                            ListRecord GrpList = TheObj.GroupList();
                            bool IsAttachObj = (TheObj.SpecialGroupKind() == attachTypeObj);
                            CheckList(GrpList,IsAttachObj,TheNSList);
                          }
                          break;
      };
    };
	};
};


void Copy3DJob::CheckObjS(ObjSpec TheObjS,NSMutableArray *TheNSList)
{
  PDObject TheObj = TheObjS;
    
  CDPoint ThePt;
  double  ObjZ;
  
  if (IsATopoObject(&TheObj, ThePt, ObjZ)) {  // see note above
    AddObjectToList(TheObj,TheNSList);
  }
  else {
    switch (TheObj.WhatObject()) {
      case pointObj   : 
      case lineObj    : 
      case rectObj    :
      case rtRectObj  :
      case rrectObj   :
      case rtArcObj   :
      case arcObj     :
      case ovalObj    :
      case rOvalObj   :
      case polyObj    :
      case smoothObj  :
      case bezObj     : AddObjectToList(TheObj,TheNSList);
                        break;
      case grpObj     :
      case xtnlGrpObj : if (mHandleGroups)
                          AddObjectToList(TheObj,TheNSList);
                        else {                       // do something about attach objects
                          ListRecord GrpList = TheObj.GroupList();
                          bool IsAttachObj = (TheObj.SpecialGroupKind() == attachTypeObj);
                          CheckList(GrpList,IsAttachObj,TheNSList);
                        }
                        break;
    };
  };

};



//----------------------------------------------------------------------------------------------